#!/usr/bin/env python2
# -*- coding: utf-8 -*-
##################################################
# GNU Radio Python Flow Graph
# Title: Satellite Receiver (Lower End)
# Description: RF Frontend (lower) part of the Blockstream Satellite receiver. This flowgraph implements the physical layer (PHY) and is the one that should run in the host where the SDR board is attached. It communicates over TCP (as server) with the upper part of the receiver, implemented in another host.
# Generated: Sat Dec  1 13:36:49 2018
##################################################

from gnuradio import blocks
from gnuradio import digital
from gnuradio import eng_notation
from gnuradio import gr
from gnuradio.eng_option import eng_option
from gnuradio.filter import firdes
from grc_gnuradio import blks2 as grc_blks2
from math import *
from optparse import OptionParser
import blocksat
import numpy
import numpy.matlib
import osmosdr
import time


class blocksat_rx_lower(gr.top_block):

    def __init__(self, cfo_poll_rate=1, fllbw=0.001, frame_sync_verbosity=1, freq=0, gain=40.0, ip='10.0.0.1', loopbw=100, n_scan_iterations=8, port=5201, scan_interval=10, scan_mode=0, sps=6):
        gr.top_block.__init__(self, "Satellite Receiver (Lower End)")

        ##################################################
        # Parameters
        ##################################################
        self.cfo_poll_rate = cfo_poll_rate
        self.fllbw = fllbw
        self.frame_sync_verbosity = frame_sync_verbosity
        self.freq = freq
        self.gain = gain
        self.ip = ip
        self.loopbw = loopbw
        self.n_scan_iterations = n_scan_iterations
        self.port = port
        self.scan_interval = scan_interval
        self.scan_mode = scan_mode
        self.sps = sps

        ##################################################
        # Variables
        ##################################################

        self.qpsk_const = qpsk_const = digital.constellation_qpsk().base()

        self.const_choice = const_choice = "qpsk"

        self.bpsk_const = bpsk_const = digital.constellation_bpsk().base()

        self.barker_code_two_dim = barker_code_two_dim = [-1.0000 - 1.0000j, -1.0000 - 1.0000j, -1.0000 - 1.0000j, -1.0000 - 1.0000j, -1.0000 - 1.0000j,  1.0000 + 1.0000j,  1.0000 + 1.0000j, -1.0000 - 1.0000j, -1.0000 - 1.0000j,  1.0000 + 1.0000j, -1.0000 - 1.0000j,  1.0000 + 1.0000j, -1.0000 - 1.0000j]
        self.barker_code_one_dim = barker_code_one_dim = sqrt(2)*numpy.real([-1.0000 - 1.0000j, -1.0000 - 1.0000j, -1.0000 - 1.0000j, -1.0000 - 1.0000j, -1.0000 - 1.0000j,  1.0000 + 1.0000j,  1.0000 + 1.0000j, -1.0000 - 1.0000j, -1.0000 - 1.0000j,  1.0000 + 1.0000j, -1.0000 - 1.0000j,  1.0000 + 1.0000j, -1.0000 - 1.0000j])
        self.n_barker_rep = n_barker_rep = 10
        self.excess_bw = excess_bw = 0.25
        self.constellation = constellation = qpsk_const if (const_choice=="qpsk") else bpsk_const
        self.barker_code = barker_code = barker_code_two_dim if (const_choice == "qpsk") else barker_code_one_dim
        self.sym_rate = sym_rate = 156250
        self.rrc_delay = rrc_delay = int(round(-44*excess_bw + 33))
        self.preamble_syms = preamble_syms = numpy.matlib.repmat(barker_code, 1, n_barker_rep)[0]
        self.nfilts = nfilts = 32
        self.n_codewords = n_codewords = 1
        self.const_order = const_order = pow(2,constellation.bits_per_symbol())
        self.codeword_len = codeword_len = 18444
        self.preamble_size = preamble_size = len(preamble_syms)
        self.payload_size = payload_size = codeword_len*n_codewords/int(numpy.log2(const_order))
        self.n_rrc_taps = n_rrc_taps = rrc_delay * int(sps*nfilts)
        self.fll_pull_in_range = fll_pull_in_range = 2 * sym_rate * (1 + 1.5*excess_bw)
        self.variable_rx_logger_0 = variable_rx_logger_0 = 0
        self.tuning_control = tuning_control = 0
        self.scan_range = scan_range = round(n_scan_iterations * fll_pull_in_range) if scan_mode else 0
        self.samp_rate = samp_rate = sym_rate * sps
        self.rrc_taps = rrc_taps = firdes.root_raised_cosine(nfilts, nfilts*sps, 1.0, excess_bw, n_rrc_taps)
        self.pmf_peak_threshold = pmf_peak_threshold = 0.7
        self.phy_preamble_overhead = phy_preamble_overhead = 1.0* preamble_size / (preamble_size + payload_size)
        self.barker_len = barker_len = 13

        ##################################################
        # Blocks
        ##################################################
        self.frame_synchronizer_0 = blocksat.frame_synchronizer(
            M=int(const_order),
            equalize=1,
            fix_phase=1,
            fw_preamble=1,
            payload_size=payload_size,
            pmf_peak_threshold=pmf_peak_threshold,
            preamble_size=preamble_size,
            preamble_syms=preamble_syms,
            verbosity=frame_sync_verbosity,
        )
        self.digital_fll_band_edge_cc_1 = digital.fll_band_edge_cc(sps, excess_bw, rrc_delay * int(sps) + 1, fllbw)
        self.blocksat_mer_measurement_pre_frame_sync = blocksat.mer_measurement(1024, int(const_order))

        self.variable_rx_logger_0 = blocksat.rx_logger(
            samp_rate,
            self.blocksat_mer_measurement_pre_frame_sync,
            1,
            self.frame_synchronizer_0,
            10,
            None,
            0,
            self.digital_fll_band_edge_cc_1,
            10,
            not scan_mode
        )


        self.tuning_control = blocksat.tuning_control(
            samp_rate,
            sps,
            scan_range,
            n_scan_iterations,
            1.0/cfo_poll_rate,
            scan_interval,
            self.frame_synchronizer_0.blocksat_frame_sync_fast_0,
            None,
            None,
            self.variable_rx_logger_0,
            None,
            self.digital_fll_band_edge_cc_1,
            self.get_freq,
            self.set_freq
        )

        self.rtlsdr_source_0 = osmosdr.source( args="numchan=" + str(1) + " " + '' )
        self.rtlsdr_source_0.set_sample_rate(samp_rate)
        self.rtlsdr_source_0.set_center_freq(freq, 0)
        self.rtlsdr_source_0.set_freq_corr(0, 0)
        self.rtlsdr_source_0.set_dc_offset_mode(0, 0)
        self.rtlsdr_source_0.set_iq_balance_mode(0, 0)
        self.rtlsdr_source_0.set_gain_mode(False, 0)
        self.rtlsdr_source_0.set_gain(gain, 0)
        self.rtlsdr_source_0.set_if_gain(20, 0)
        self.rtlsdr_source_0.set_bb_gain(20, 0)
        self.rtlsdr_source_0.set_antenna('', 0)
        self.rtlsdr_source_0.set_bandwidth(0, 0)

        self.digital_pfb_clock_sync_xxx_0 = digital.pfb_clock_sync_ccf(sps, 2*pi/50, (rrc_taps), nfilts, nfilts/2, pi/8, 1)
        self.digital_costas_loop_cc_0 = digital.costas_loop_cc(2*pi/loopbw, 2**constellation.bits_per_symbol(), False)
        self.blocksat_da_carrier_phase_rec_0_0 = blocksat.da_carrier_phase_rec(((1/sqrt(2))*preamble_syms), 0.001, 1/sqrt(2), int(const_order), True, True)
        self.blocks_rms_xx_1 = blocks.rms_cf(0.0001)
        self.blocks_null_sink_0_0_0 = blocks.null_sink(gr.sizeof_float*1)
        self.blocks_null_sink_0_0 = blocks.null_sink(gr.sizeof_float*1)
        self.blocks_null_sink_0 = blocks.null_sink(gr.sizeof_float*1)
        self.blocks_float_to_complex_0 = blocks.float_to_complex(1)
        self.blocks_divide_xx_0 = blocks.divide_cc(1)
        self.blks2_tcp_sink_0 = grc_blks2.tcp_sink(
        	itemsize=gr.sizeof_gr_complex*1,
        	addr=ip,
        	port=port,
        	server=True,
        )

        ##################################################
        # Connections
        ##################################################
        self.connect((self.blocks_divide_xx_0, 0), (self.digital_fll_band_edge_cc_1, 0))
        self.connect((self.blocks_float_to_complex_0, 0), (self.blocks_divide_xx_0, 1))
        self.connect((self.blocks_rms_xx_1, 0), (self.blocks_float_to_complex_0, 0))
        self.connect((self.blocksat_da_carrier_phase_rec_0_0, 0), (self.blks2_tcp_sink_0, 0))
        self.connect((self.blocksat_da_carrier_phase_rec_0_0, 1), (self.blocks_null_sink_0_0, 0))
        self.connect((self.blocksat_mer_measurement_pre_frame_sync, 0), (self.blocks_null_sink_0, 0))
        self.connect((self.digital_costas_loop_cc_0, 0), (self.blocksat_mer_measurement_pre_frame_sync, 0))
        self.connect((self.digital_costas_loop_cc_0, 0), (self.frame_synchronizer_0, 0))
        self.connect((self.digital_fll_band_edge_cc_1, 0), (self.digital_pfb_clock_sync_xxx_0, 0))
        self.connect((self.digital_pfb_clock_sync_xxx_0, 0), (self.digital_costas_loop_cc_0, 0))
        self.connect((self.frame_synchronizer_0, 2), (self.blocks_null_sink_0_0_0, 0))
        self.connect((self.frame_synchronizer_0, 1), (self.blocksat_da_carrier_phase_rec_0_0, 1))
        self.connect((self.frame_synchronizer_0, 0), (self.blocksat_da_carrier_phase_rec_0_0, 0))
        self.connect((self.rtlsdr_source_0, 0), (self.blocks_divide_xx_0, 0))
        self.connect((self.rtlsdr_source_0, 0), (self.blocks_rms_xx_1, 0))

    def get_cfo_poll_rate(self):
        return self.cfo_poll_rate

    def set_cfo_poll_rate(self, cfo_poll_rate):
        self.cfo_poll_rate = cfo_poll_rate

    def get_fllbw(self):
        return self.fllbw

    def set_fllbw(self, fllbw):
        self.fllbw = fllbw
        self.digital_fll_band_edge_cc_1.set_loop_bandwidth(self.fllbw)

    def get_frame_sync_verbosity(self):
        return self.frame_sync_verbosity

    def set_frame_sync_verbosity(self, frame_sync_verbosity):
        self.frame_sync_verbosity = frame_sync_verbosity
        self.frame_synchronizer_0.set_verbosity(self.frame_sync_verbosity)

    def get_freq(self):
        return self.freq

    def set_freq(self, freq):
        self.freq = freq
        self.rtlsdr_source_0.set_center_freq(self.freq, 0)

    def get_gain(self):
        return self.gain

    def set_gain(self, gain):
        self.gain = gain
        self.rtlsdr_source_0.set_gain(self.gain, 0)

    def get_ip(self):
        return self.ip

    def set_ip(self, ip):
        self.ip = ip

    def get_loopbw(self):
        return self.loopbw

    def set_loopbw(self, loopbw):
        self.loopbw = loopbw
        self.digital_costas_loop_cc_0.set_loop_bandwidth(2*pi/self.loopbw)

    def get_n_scan_iterations(self):
        return self.n_scan_iterations

    def set_n_scan_iterations(self, n_scan_iterations):
        self.n_scan_iterations = n_scan_iterations
        self.set_scan_range(round(self.n_scan_iterations * self.fll_pull_in_range) if self.scan_mode else 0)

    def get_port(self):
        return self.port

    def set_port(self, port):
        self.port = port

    def get_scan_interval(self):
        return self.scan_interval

    def set_scan_interval(self, scan_interval):
        self.scan_interval = scan_interval

    def get_scan_mode(self):
        return self.scan_mode

    def set_scan_mode(self, scan_mode):
        self.scan_mode = scan_mode
        self.set_scan_range(round(self.n_scan_iterations * self.fll_pull_in_range) if self.scan_mode else 0)

    def get_sps(self):
        return self.sps

    def set_sps(self, sps):
        self.sps = sps
        self.set_samp_rate(self.sym_rate * self.sps)
        self.set_rrc_taps(firdes.root_raised_cosine(self.nfilts, self.nfilts*self.sps, 1.0, self.excess_bw, self.n_rrc_taps))
        self.set_n_rrc_taps(self.rrc_delay * int(self.sps*self.nfilts))

    def get_qpsk_const(self):
        return self.qpsk_const

    def set_qpsk_const(self, qpsk_const):
        self.qpsk_const = qpsk_const
        self.set_constellation(self.qpsk_const if (self.const_choice=="qpsk") else self.bpsk_const)

    def get_const_choice(self):
        return self.const_choice

    def set_const_choice(self, const_choice):
        self.const_choice = const_choice
        self.set_constellation(self.qpsk_const if (self.const_choice=="qpsk") else self.bpsk_const)
        self.set_barker_code(self.barker_code_two_dim if (self.const_choice == "qpsk") else self.barker_code_one_dim)

    def get_bpsk_const(self):
        return self.bpsk_const

    def set_bpsk_const(self, bpsk_const):
        self.bpsk_const = bpsk_const
        self.set_constellation(self.qpsk_const if (self.const_choice=="qpsk") else self.bpsk_const)

    def get_barker_code_two_dim(self):
        return self.barker_code_two_dim

    def set_barker_code_two_dim(self, barker_code_two_dim):
        self.barker_code_two_dim = barker_code_two_dim
        self.set_barker_code(self.barker_code_two_dim if (self.const_choice == "qpsk") else self.barker_code_one_dim)

    def get_barker_code_one_dim(self):
        return self.barker_code_one_dim

    def set_barker_code_one_dim(self, barker_code_one_dim):
        self.barker_code_one_dim = barker_code_one_dim
        self.set_barker_code(self.barker_code_two_dim if (self.const_choice == "qpsk") else self.barker_code_one_dim)

    def get_n_barker_rep(self):
        return self.n_barker_rep

    def set_n_barker_rep(self, n_barker_rep):
        self.n_barker_rep = n_barker_rep
        self.set_preamble_syms(numpy.matlib.repmat(self.barker_code, 1, self.n_barker_rep)[0])

    def get_excess_bw(self):
        return self.excess_bw

    def set_excess_bw(self, excess_bw):
        self.excess_bw = excess_bw
        self.set_rrc_delay(int(round(-44*self.excess_bw + 33)))
        self.set_rrc_taps(firdes.root_raised_cosine(self.nfilts, self.nfilts*self.sps, 1.0, self.excess_bw, self.n_rrc_taps))
        self.set_fll_pull_in_range(2 * self.sym_rate * (1 + 1.5*self.excess_bw))

    def get_constellation(self):
        return self.constellation

    def set_constellation(self, constellation):
        self.constellation = constellation

    def get_barker_code(self):
        return self.barker_code

    def set_barker_code(self, barker_code):
        self.barker_code = barker_code
        self.set_preamble_syms(numpy.matlib.repmat(self.barker_code, 1, self.n_barker_rep)[0])

    def get_sym_rate(self):
        return self.sym_rate

    def set_sym_rate(self, sym_rate):
        self.sym_rate = sym_rate
        self.set_samp_rate(self.sym_rate * self.sps)
        self.set_fll_pull_in_range(2 * self.sym_rate * (1 + 1.5*self.excess_bw))

    def get_rrc_delay(self):
        return self.rrc_delay

    def set_rrc_delay(self, rrc_delay):
        self.rrc_delay = rrc_delay
        self.set_n_rrc_taps(self.rrc_delay * int(self.sps*self.nfilts))

    def get_preamble_syms(self):
        return self.preamble_syms

    def set_preamble_syms(self, preamble_syms):
        self.preamble_syms = preamble_syms
        self.set_preamble_size(len(self.preamble_syms))
        self.frame_synchronizer_0.set_preamble_syms(self.preamble_syms)

    def get_nfilts(self):
        return self.nfilts

    def set_nfilts(self, nfilts):
        self.nfilts = nfilts
        self.set_rrc_taps(firdes.root_raised_cosine(self.nfilts, self.nfilts*self.sps, 1.0, self.excess_bw, self.n_rrc_taps))
        self.set_n_rrc_taps(self.rrc_delay * int(self.sps*self.nfilts))

    def get_n_codewords(self):
        return self.n_codewords

    def set_n_codewords(self, n_codewords):
        self.n_codewords = n_codewords
        self.set_payload_size(self.codeword_len*self.n_codewords/int(numpy.log2(self.const_order)))

    def get_const_order(self):
        return self.const_order

    def set_const_order(self, const_order):
        self.const_order = const_order
        self.set_payload_size(self.codeword_len*self.n_codewords/int(numpy.log2(self.const_order)))
        self.frame_synchronizer_0.set_M(int(self.const_order))

    def get_codeword_len(self):
        return self.codeword_len

    def set_codeword_len(self, codeword_len):
        self.codeword_len = codeword_len
        self.set_payload_size(self.codeword_len*self.n_codewords/int(numpy.log2(self.const_order)))

    def get_preamble_size(self):
        return self.preamble_size

    def set_preamble_size(self, preamble_size):
        self.preamble_size = preamble_size
        self.frame_synchronizer_0.set_preamble_size(self.preamble_size)
        self.set_phy_preamble_overhead(1.0* self.preamble_size / (self.preamble_size + self.payload_size))

    def get_payload_size(self):
        return self.payload_size

    def set_payload_size(self, payload_size):
        self.payload_size = payload_size
        self.frame_synchronizer_0.set_payload_size(self.payload_size)
        self.set_phy_preamble_overhead(1.0* self.preamble_size / (self.preamble_size + self.payload_size))

    def get_n_rrc_taps(self):
        return self.n_rrc_taps

    def set_n_rrc_taps(self, n_rrc_taps):
        self.n_rrc_taps = n_rrc_taps
        self.set_rrc_taps(firdes.root_raised_cosine(self.nfilts, self.nfilts*self.sps, 1.0, self.excess_bw, self.n_rrc_taps))

    def get_fll_pull_in_range(self):
        return self.fll_pull_in_range

    def set_fll_pull_in_range(self, fll_pull_in_range):
        self.fll_pull_in_range = fll_pull_in_range
        self.set_scan_range(round(self.n_scan_iterations * self.fll_pull_in_range) if self.scan_mode else 0)

    def get_variable_rx_logger_0(self):
        return self.variable_rx_logger_0

    def set_variable_rx_logger_0(self, variable_rx_logger_0):
        self.variable_rx_logger_0 = variable_rx_logger_0

    def get_tuning_control(self):
        return self.tuning_control

    def set_tuning_control(self, tuning_control):
        self.tuning_control = tuning_control

    def get_scan_range(self):
        return self.scan_range

    def set_scan_range(self, scan_range):
        self.scan_range = scan_range

    def get_samp_rate(self):
        return self.samp_rate

    def set_samp_rate(self, samp_rate):
        self.samp_rate = samp_rate
        self.rtlsdr_source_0.set_sample_rate(self.samp_rate)

    def get_rrc_taps(self):
        return self.rrc_taps

    def set_rrc_taps(self, rrc_taps):
        self.rrc_taps = rrc_taps
        self.digital_pfb_clock_sync_xxx_0.update_taps((self.rrc_taps))

    def get_pmf_peak_threshold(self):
        return self.pmf_peak_threshold

    def set_pmf_peak_threshold(self, pmf_peak_threshold):
        self.pmf_peak_threshold = pmf_peak_threshold
        self.frame_synchronizer_0.set_pmf_peak_threshold(self.pmf_peak_threshold)

    def get_phy_preamble_overhead(self):
        return self.phy_preamble_overhead

    def set_phy_preamble_overhead(self, phy_preamble_overhead):
        self.phy_preamble_overhead = phy_preamble_overhead

    def get_barker_len(self):
        return self.barker_len

    def set_barker_len(self, barker_len):
        self.barker_len = barker_len


def argument_parser():
    description = 'RF Frontend (lower) part of the Blockstream Satellite receiver. This flowgraph implements the physical layer (PHY) and is the one that should run in the host where the SDR board is attached. It communicates over TCP (as server) with the upper part of the receiver, implemented in another host.'
    parser = OptionParser(usage="%prog: [options]", option_class=eng_option, description=description)
    parser.add_option(
        "-c", "--cfo-poll-rate", dest="cfo_poll_rate", type="eng_float", default=eng_notation.num_to_str(1),
        help="Set CFO Poll Rate (Hz) [default=%default]")
    parser.add_option(
        "-b", "--fllbw", dest="fllbw", type="eng_float", default=eng_notation.num_to_str(0.001),
        help="Set Frequency-locked Loop Bandwidth [default=%default]")
    parser.add_option(
        "-v", "--frame-sync-verbosity", dest="frame_sync_verbosity", type="intx", default=1,
        help="Set Frame Sync Verbosity [default=%default]")
    parser.add_option(
        "-f", "--freq", dest="freq", type="intx", default=0,
        help="Set RF Center Frequency [default=%default]")
    parser.add_option(
        "-g", "--gain", dest="gain", type="eng_float", default=eng_notation.num_to_str(40.0),
        help="Set Rx Gain [default=%default]")
    parser.add_option(
        "-i", "--ip", dest="ip", type="string", default='10.0.0.1',
        help="Set Local IP Address [default=%default]")
    parser.add_option(
        "-l", "--loopbw", dest="loopbw", type="intx", default=100,
        help="Set Costas Loop Bandwidth [default=%default]")
    parser.add_option(
        "-n", "--n-scan-iterations", dest="n_scan_iterations", type="intx", default=8,
        help="Set Number of Frequency Scan Iterations  [default=%default]")
    parser.add_option(
        "-p", "--port", dest="port", type="intx", default=5201,
        help="Set Destination Port [default=%default]")
    parser.add_option(
        "-t", "--scan-interval", dest="scan_interval", type="eng_float", default=eng_notation.num_to_str(10),
        help="Set Frequency Scan Interval (sec) [default=%default]")
    parser.add_option(
        "-s", "--scan-mode", dest="scan_mode", action="store_true", default=False,
        help="Set Frequency Scan Mode [default=%default]")
    parser.add_option(
        "-o", "--sps", dest="sps", type="intx", default=6,
        help="Set Oversampling Ratio [default=%default]")
    return parser


def main(top_block_cls=blocksat_rx_lower, options=None):
    if options is None:
        options, _ = argument_parser().parse_args()

    tb = top_block_cls(cfo_poll_rate=options.cfo_poll_rate, fllbw=options.fllbw, frame_sync_verbosity=options.frame_sync_verbosity, freq=options.freq, gain=options.gain, ip=options.ip, loopbw=options.loopbw, n_scan_iterations=options.n_scan_iterations, port=options.port, scan_interval=options.scan_interval, scan_mode=options.scan_mode, sps=options.sps)
    tb.start()
    try:
        raw_input('Press Enter to quit: ')
    except EOFError:
        pass
    tb.stop()
    tb.wait()


if __name__ == '__main__':
    main()
