#!/usr/bin/env python2
# -*- coding: utf-8 -*-
##################################################
# GNU Radio Python Flow Graph
# Title: Satellite Receiver (Upper End)
# Description: Upper part of the Blockstream Satellite receiver. This flowgraph implements the decoder and the MAC layer of the receiver. It is the one that should run in the host where the FIBRE application is executed. It communicates via TCP transport (as the client) with the lower (RF frontend) part of the receiver.
# Generated: Sat Dec  1 13:36:55 2018
##################################################

from gnuradio import blocks
from gnuradio import digital
from gnuradio import eng_notation
from gnuradio import gr
from gnuradio.eng_option import eng_option
from gnuradio.filter import firdes
from grc_gnuradio import blks2 as grc_blks2
from math import *
from optparse import OptionParser
import blocksat
import framers
import numpy
import numpy.matlib
import pmt


class blocksat_rx_upper(gr.top_block):

    def __init__(self, port=5201, src_ip='10.0.0.1', version=1, no_api=0):
        gr.top_block.__init__(self, "Satellite Receiver (Upper End)")

        ##################################################
        # Parameters
        ##################################################
        self.port = port
        self.src_ip = src_ip
        self.version = version
        self.no_api = no_api

        ##################################################
        # Variables
        ##################################################

        self.qpsk_const = qpsk_const = digital.constellation_qpsk().base()

        self.const_choice = const_choice = "qpsk"

        self.bpsk_const = bpsk_const = digital.constellation_bpsk().base()

        self.barker_code_two_dim = barker_code_two_dim = [-1.0000 - 1.0000j, -1.0000 - 1.0000j, -1.0000 - 1.0000j, -1.0000 - 1.0000j, -1.0000 - 1.0000j,  1.0000 + 1.0000j,  1.0000 + 1.0000j, -1.0000 - 1.0000j, -1.0000 - 1.0000j,  1.0000 + 1.0000j, -1.0000 - 1.0000j,  1.0000 + 1.0000j, -1.0000 - 1.0000j]
        self.barker_code_one_dim = barker_code_one_dim = sqrt(2)*numpy.real([-1.0000 - 1.0000j, -1.0000 - 1.0000j, -1.0000 - 1.0000j, -1.0000 - 1.0000j, -1.0000 - 1.0000j,  1.0000 + 1.0000j,  1.0000 + 1.0000j, -1.0000 - 1.0000j, -1.0000 - 1.0000j,  1.0000 + 1.0000j, -1.0000 - 1.0000j,  1.0000 + 1.0000j, -1.0000 - 1.0000j])
        self.n_barker_rep = n_barker_rep = 10
        self.constellation = constellation = qpsk_const if (const_choice=="qpsk") else bpsk_const
        self.barker_code = barker_code = barker_code_two_dim if (const_choice == "qpsk") else barker_code_one_dim
        self.preamble_syms = preamble_syms = numpy.matlib.repmat(barker_code, 1, n_barker_rep)[0]
        self.n_codewords = n_codewords = 1
        self.const_order = const_order = pow(2,constellation.bits_per_symbol())
        self.codeword_len = codeword_len = 18444
        self.preamble_size = preamble_size = len(preamble_syms)
        self.payload_size = payload_size = codeword_len*n_codewords/int(numpy.log2(const_order))
        self.dataword_len = dataword_len = 6144
        self.sym_rate = sym_rate = 156250
        self.phy_preamble_overhead = phy_preamble_overhead = 1.0* preamble_size / (preamble_size + payload_size)
        self.code_rate = code_rate = 1.0*dataword_len/codeword_len
        self.variable_rx_logger_0 = variable_rx_logger_0 = 0
        self.phy_bit_rate = phy_bit_rate = sym_rate* ( constellation.bits_per_symbol() ) * (code_rate) * (1.-phy_preamble_overhead)
        self.barker_len = barker_len = 13

        ##################################################
        # Blocks
        ##################################################
        self.framers_gr_hdlc_deframer_b_0 = framers.gr_hdlc_deframer_b(0)

        self.variable_rx_logger_0 = blocksat.rx_logger(
            None,
            None,
            0,
            None,
            0,
            self.framers_gr_hdlc_deframer_b_0,
            1,
            None,
            0,
            True
        )

        self.digital_map_bb_0_0_0 = digital.map_bb(([1,- 1]))
        self.digital_descrambler_bb_0 = digital.descrambler_bb(0x21, 0x7F, 16)
        self.digital_constellation_decoder_cb_0 = digital.constellation_decoder_cb(constellation.base())
        self.blocksat_turbo_decoder_0 = blocksat.turbo_decoder(codeword_len, dataword_len)
        self.blocksat_protocol_sink_0 = blocksat.protocol_sink('/tmp/blocksat/bitcoinfibre', '/tmp/blocksat/api', version, bool(no_api))
        self.blocks_unpack_k_bits_bb_0 = blocks.unpack_k_bits_bb(constellation.bits_per_symbol())
        self.blocks_throttle_0 = blocks.throttle(gr.sizeof_gr_complex*1, sym_rate,True)
        self.blks2_tcp_source_0 = grc_blks2.tcp_source(
        	itemsize=gr.sizeof_gr_complex*1,
        	addr=src_ip,
        	port=port,
        	server=False,
        )

        ##################################################
        # Connections
        ##################################################
        self.msg_connect((self.framers_gr_hdlc_deframer_b_0, 'pdu'), (self.blocksat_protocol_sink_0, 'async_pdu'))
        self.connect((self.blks2_tcp_source_0, 0), (self.blocks_throttle_0, 0))
        self.connect((self.blocks_throttle_0, 0), (self.digital_constellation_decoder_cb_0, 0))
        self.connect((self.blocks_unpack_k_bits_bb_0, 0), (self.digital_map_bb_0_0_0, 0))
        self.connect((self.blocksat_turbo_decoder_0, 0), (self.digital_descrambler_bb_0, 0))
        self.connect((self.digital_constellation_decoder_cb_0, 0), (self.blocks_unpack_k_bits_bb_0, 0))
        self.connect((self.digital_descrambler_bb_0, 0), (self.framers_gr_hdlc_deframer_b_0, 0))
        self.connect((self.digital_map_bb_0_0_0, 0), (self.blocksat_turbo_decoder_0, 0))

    def get_port(self):
        return self.port

    def set_port(self, port):
        self.port = port

    def get_src_ip(self):
        return self.src_ip

    def set_src_ip(self, src_ip):
        self.src_ip = src_ip

    def get_version(self):
        return self.version

    def set_version(self, version):
        self.version = version

    def get_no_api(self):
        return self.no_api

    def set_no_api(self, no_api):
        self.no_api = no_api

    def get_qpsk_const(self):
        return self.qpsk_const

    def set_qpsk_const(self, qpsk_const):
        self.qpsk_const = qpsk_const
        self.set_constellation(self.qpsk_const if (self.const_choice=="qpsk") else self.bpsk_const)

    def get_const_choice(self):
        return self.const_choice

    def set_const_choice(self, const_choice):
        self.const_choice = const_choice
        self.set_constellation(self.qpsk_const if (self.const_choice=="qpsk") else self.bpsk_const)
        self.set_barker_code(self.barker_code_two_dim if (self.const_choice == "qpsk") else self.barker_code_one_dim)

    def get_bpsk_const(self):
        return self.bpsk_const

    def set_bpsk_const(self, bpsk_const):
        self.bpsk_const = bpsk_const
        self.set_constellation(self.qpsk_const if (self.const_choice=="qpsk") else self.bpsk_const)

    def get_barker_code_two_dim(self):
        return self.barker_code_two_dim

    def set_barker_code_two_dim(self, barker_code_two_dim):
        self.barker_code_two_dim = barker_code_two_dim
        self.set_barker_code(self.barker_code_two_dim if (self.const_choice == "qpsk") else self.barker_code_one_dim)

    def get_barker_code_one_dim(self):
        return self.barker_code_one_dim

    def set_barker_code_one_dim(self, barker_code_one_dim):
        self.barker_code_one_dim = barker_code_one_dim
        self.set_barker_code(self.barker_code_two_dim if (self.const_choice == "qpsk") else self.barker_code_one_dim)

    def get_n_barker_rep(self):
        return self.n_barker_rep

    def set_n_barker_rep(self, n_barker_rep):
        self.n_barker_rep = n_barker_rep
        self.set_preamble_syms(numpy.matlib.repmat(self.barker_code, 1, self.n_barker_rep)[0])

    def get_constellation(self):
        return self.constellation

    def set_constellation(self, constellation):
        self.constellation = constellation

    def get_barker_code(self):
        return self.barker_code

    def set_barker_code(self, barker_code):
        self.barker_code = barker_code
        self.set_preamble_syms(numpy.matlib.repmat(self.barker_code, 1, self.n_barker_rep)[0])

    def get_preamble_syms(self):
        return self.preamble_syms

    def set_preamble_syms(self, preamble_syms):
        self.preamble_syms = preamble_syms
        self.set_preamble_size(len(self.preamble_syms))

    def get_n_codewords(self):
        return self.n_codewords

    def set_n_codewords(self, n_codewords):
        self.n_codewords = n_codewords
        self.set_payload_size(self.codeword_len*self.n_codewords/int(numpy.log2(self.const_order)))

    def get_const_order(self):
        return self.const_order

    def set_const_order(self, const_order):
        self.const_order = const_order
        self.set_payload_size(self.codeword_len*self.n_codewords/int(numpy.log2(self.const_order)))

    def get_codeword_len(self):
        return self.codeword_len

    def set_codeword_len(self, codeword_len):
        self.codeword_len = codeword_len
        self.set_payload_size(self.codeword_len*self.n_codewords/int(numpy.log2(self.const_order)))
        self.set_code_rate(1.0*self.dataword_len/self.codeword_len)

    def get_preamble_size(self):
        return self.preamble_size

    def set_preamble_size(self, preamble_size):
        self.preamble_size = preamble_size
        self.set_phy_preamble_overhead(1.0* self.preamble_size / (self.preamble_size + self.payload_size))

    def get_payload_size(self):
        return self.payload_size

    def set_payload_size(self, payload_size):
        self.payload_size = payload_size
        self.set_phy_preamble_overhead(1.0* self.preamble_size / (self.preamble_size + self.payload_size))

    def get_dataword_len(self):
        return self.dataword_len

    def set_dataword_len(self, dataword_len):
        self.dataword_len = dataword_len
        self.set_code_rate(1.0*self.dataword_len/self.codeword_len)

    def get_sym_rate(self):
        return self.sym_rate

    def set_sym_rate(self, sym_rate):
        self.sym_rate = sym_rate
        self.set_phy_bit_rate(self.sym_rate* ( constellation.bits_per_symbol() ) * (self.code_rate) * (1.-self.phy_preamble_overhead))
        self.blocks_throttle_0.set_sample_rate(self.sym_rate)

    def get_phy_preamble_overhead(self):
        return self.phy_preamble_overhead

    def set_phy_preamble_overhead(self, phy_preamble_overhead):
        self.phy_preamble_overhead = phy_preamble_overhead
        self.set_phy_bit_rate(self.sym_rate* ( constellation.bits_per_symbol() ) * (self.code_rate) * (1.-self.phy_preamble_overhead))

    def get_code_rate(self):
        return self.code_rate

    def set_code_rate(self, code_rate):
        self.code_rate = code_rate
        self.set_phy_bit_rate(self.sym_rate* ( constellation.bits_per_symbol() ) * (self.code_rate) * (1.-self.phy_preamble_overhead))

    def get_variable_rx_logger_0(self):
        return self.variable_rx_logger_0

    def set_variable_rx_logger_0(self, variable_rx_logger_0):
        self.variable_rx_logger_0 = variable_rx_logger_0

    def get_phy_bit_rate(self):
        return self.phy_bit_rate

    def set_phy_bit_rate(self, phy_bit_rate):
        self.phy_bit_rate = phy_bit_rate

    def get_barker_len(self):
        return self.barker_len

    def set_barker_len(self, barker_len):
        self.barker_len = barker_len


def argument_parser():
    description = 'Upper part of the Blockstream Satellite receiver. This flowgraph implements the decoder and the MAC layer of the receiver. It is the one that should run in the host where the FIBRE application is executed. It communicates via TCP transport (as the client) with the lower (RF frontend) part of the receiver.'
    parser = OptionParser(usage="%prog: [options]", option_class=eng_option, description=description)
    parser.add_option(
        "-p", "--port", dest="port", type="intx", default=5201,
        help="Set TCP Port [default=%default]")
    parser.add_option(
        "-i", "--src-ip", dest="src_ip", type="string", default='10.0.0.1',
        help="Set Source IP Address (Rx Lower IP) [default=%default]")
    parser.add_option(
        "", "--version", dest="version", type="intx", default=1,
        help="Set Blockstream Satellite Protocol Version [default=%default]")
    parser.add_option(
        "", "--no-api", dest="no_api", action="store_true", default=False,
        help="Set Disable User API Data Output [default=%default]")
    return parser


def main(top_block_cls=blocksat_rx_upper, options=None):
    if options is None:
        options, _ = argument_parser().parse_args()

    tb = top_block_cls(port=options.port, src_ip=options.src_ip, version=options.version, no_api=options.no_api)
    tb.start()
    try:
        raw_input('Press Enter to quit: ')
    except EOFError:
        pass
    tb.stop()
    tb.wait()


if __name__ == '__main__':
    main()
